@@ -1,14 +1,16 @@
 from pathlib import Path
 from unittest.mock import MagicMock, patch
 
+import pytest
+
 from sync_pre_commit_lock import Printer
 from sync_pre_commit_lock.actions.sync_hooks import (
     GenericLockedPackage,
     SyncPreCommitHooksVersion,
 )
 from sync_pre_commit_lock.config import SyncPreCommitLockConfig
-from sync_pre_commit_lock.db import PackageRepoMapping, RepoInfo
-from sync_pre_commit_lock.pre_commit_config import PreCommitHookConfig, PreCommitRepo
+from sync_pre_commit_lock.db import RepoInfo
+from sync_pre_commit_lock.pre_commit_config import PreCommitHook, PreCommitHookConfig, PreCommitRepo
 
 
 def test_execute_returns_early_when_disabled() -> None:
@@ -31,11 +33,8 @@ def test_execute_returns_early_when_disabled() -> None:
 
 
 @patch("sync_pre_commit_lock.pre_commit_config.PreCommitHookConfig.from_yaml_file")
-@patch.object(SyncPreCommitHooksVersion, "build_mapping")
 @patch.object(SyncPreCommitHooksVersion, "analyze_repos")
-def test_execute_returns_early_during_dry_run(
-    mock_analyze_repos: MagicMock, mock_build_mapping: MagicMock, mock_from_yaml_file: MagicMock
-) -> None:
+def test_execute_returns_early_during_dry_run(mock_analyze_repos: MagicMock, mock_from_yaml_file: MagicMock) -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
     locked_packages: dict[str, GenericLockedPackage] = {}
@@ -54,13 +53,12 @@ def test_execute_returns_early_during_dry_run(
     # Mocks
     pre_commit_config = MagicMock(spec=PreCommitHookConfig)
     mock_from_yaml_file.return_value = pre_commit_config
-    mock_build_mapping.return_value = ({}, {"repo1": "somepkg"})
+    syncer.mapping_reverse_by_url = {"repo1": "somepkg"}
     mock_analyze_repos.return_value = {PreCommitRepo("repo1", "rev1"): "rev2"}, {}
 
     syncer.execute()
 
     # Assertions
-    mock_build_mapping.assert_called_once()
     mock_analyze_repos.assert_called_once()
     pre_commit_config.update_pre_commit_repo_versions.assert_not_called()
     printer.info.assert_called_with("Dry run, skipping pre-commit hook update.")
@@ -109,11 +107,8 @@ def test_execute_handles_file_invalid(mock_from_yaml_file: MagicMock) -> None:
 
 
 @patch("sync_pre_commit_lock.pre_commit_config.PreCommitHookConfig.from_yaml_file")
-@patch.object(SyncPreCommitHooksVersion, "build_mapping")
 @patch.object(SyncPreCommitHooksVersion, "analyze_repos")
-def test_execute_synchronizes_hooks(
-    mock_analyze_repos: MagicMock, mock_build_mapping: MagicMock, mock_from_yaml_file: MagicMock
-) -> None:
+def test_execute_synchronizes_hooks(mock_analyze_repos: MagicMock, mock_from_yaml_file: MagicMock) -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
     pre_commit_config_file_path.name = ".pre-commit-config.yaml"
@@ -133,24 +128,20 @@ def test_execute_synchronizes_hooks(
     # Mocks
     pre_commit_config = MagicMock(spec=PreCommitHookConfig)
     mock_from_yaml_file.return_value = pre_commit_config
-    mock_build_mapping.return_value = ({}, {"repo1": "somepkg"})
+    syncer.mapping_reverse_by_url = {"repo1": "somepkg"}
     mock_analyze_repos.return_value = {PreCommitRepo("repo1", "rev1"): "rev2"}, {}
 
     syncer.execute()
 
     # Assertions
-    mock_build_mapping.assert_called_once()
     mock_analyze_repos.assert_called_once()
     pre_commit_config.update_pre_commit_repo_versions.assert_called_once_with({PreCommitRepo("repo1", "rev1"): "rev2"})
     printer.success.assert_called_with("Pre-commit hooks have been updated in .pre-commit-config.yaml!")
 
 
 @patch("sync_pre_commit_lock.pre_commit_config.PreCommitHookConfig.from_yaml_file")
-@patch.object(SyncPreCommitHooksVersion, "build_mapping")
 @patch.object(SyncPreCommitHooksVersion, "analyze_repos")
-def test_execute_synchronizes_hooks_no_match(
-    mock_analyze_repos: MagicMock, mock_build_mapping: MagicMock, mock_from_yaml_file: MagicMock
-) -> None:
+def test_execute_synchronizes_hooks_no_match(mock_analyze_repos: MagicMock, mock_from_yaml_file: MagicMock) -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
     locked_packages: dict[str, GenericLockedPackage] = {}
@@ -165,17 +156,16 @@ def test_execute_synchronizes_hooks_no_match(
         plugin_config=plugin_config,
         dry_run=dry_run,
     )
+    syncer.mapping = {}
 
     # Mocks
     pre_commit_config = MagicMock(spec=PreCommitHookConfig)
     mock_from_yaml_file.return_value = pre_commit_config
-    mock_build_mapping.return_value = ({}, {})
     mock_analyze_repos.return_value = {}, {}
 
     syncer.execute()
 
     # Assertions
-    mock_build_mapping.assert_called_once()
     mock_analyze_repos.assert_called_once()
     pre_commit_config.update_pre_commit_repo_versions.assert_not_called()
     printer.info.assert_called_with("No pre-commit hook detected that matches a locked package.")
@@ -184,7 +174,7 @@ def test_execute_synchronizes_hooks_no_match(
 def test_get_pre_commit_repo_new_version() -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
-    locked_packages: dict[str, GenericLockedPackage] = {"lib_name": GenericLockedPackage("lib_name", "2.0.0")}
+    locked_packages: dict[str, GenericLockedPackage] = {"lib-name": GenericLockedPackage("lib-name", "2.0.0")}
     plugin_config = MagicMock(spec=SyncPreCommitLockConfig)
     plugin_config.ignore = []
     syncer = SyncPreCommitHooksVersion(
@@ -194,11 +184,9 @@ def test_get_pre_commit_repo_new_version() -> None:
         plugin_config=plugin_config,
     )
     pre_commit_config_repo = PreCommitRepo("repo_url", "1.2.3")
-    mapping_db_repo_info: RepoInfo = {"repo": "repo_url", "rev": "${rev}"}
+    syncer.mapping = {"lib-name": {"repo": "repo_url", "rev": "${rev}"}}
 
-    new_version = syncer.get_pre_commit_repo_new_version(
-        pre_commit_config_repo, mapping_db_repo_info, locked_packages["lib_name"]
-    )
+    new_version = syncer.get_pre_commit_repo_new_version(pre_commit_config_repo)
 
     assert new_version == "2.0.0"
 
@@ -207,7 +195,7 @@ def test_get_pre_commit_repo_new_version() -> None:
 def test_analyze_repos(mock_get_pre_commit_repo_new_version: MagicMock) -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
-    locked_packages: dict[str, GenericLockedPackage] = {"lib_name": GenericLockedPackage("lib_name", "2.0.0")}
+    locked_packages: dict[str, GenericLockedPackage] = {"lib-name": GenericLockedPackage("lib-name", "2.0.0")}
     plugin_config = MagicMock(spec=SyncPreCommitLockConfig)
 
     syncer = SyncPreCommitHooksVersion(
@@ -218,12 +206,12 @@ def test_analyze_repos(mock_get_pre_commit_repo_new_version: MagicMock) -> None:
     )
     mock_get_pre_commit_repo_new_version.return_value = "2.0.0"
     pre_commit_repos = {PreCommitRepo("https://repo_url", "1.2.3")}
-    mapping: PackageRepoMapping = {"lib_name": {"repo": "https://repo_url", "rev": "${rev}"}}
-    mapping_reverse_by_url = {"https://repo_url": "lib_name"}
+    syncer.mapping = {"lib-name": {"repo": "https://repo_url", "rev": "${rev}"}}
+    syncer.mapping_reverse_by_url = {"https://repo_url": "lib-name"}
 
-    to_fix, _ = syncer.analyze_repos(pre_commit_repos, mapping, mapping_reverse_by_url)
+    to_fix, _ = syncer.analyze_repos(pre_commit_repos)
 
-    assert to_fix == {PreCommitRepo("https://repo_url", "1.2.3"): "2.0.0"}
+    assert to_fix == {PreCommitRepo("https://repo_url", "1.2.3"): PreCommitRepo("https://repo_url", "2.0.0")}
 
 
 def test_build_mapping() -> None:
@@ -240,20 +228,18 @@ def test_build_mapping() -> None:
     )
     plugin_config.dependency_mapping = {"new_lib": {"repo": "new_repo_url", "rev": "${rev}"}}
 
-    mapping, mapping_reverse_by_url = syncer.build_mapping()
-
-    assert "new_lib" in mapping
-    assert mapping["new_lib"]["repo"] == "new_repo_url"
-    assert "new_repo_url" in mapping_reverse_by_url
-    assert mapping_reverse_by_url["new_repo_url"] == "new_lib"
+    assert "new_lib" in syncer.mapping
+    assert syncer.mapping["new_lib"]["repo"] == "new_repo_url"
+    assert "new_repo_url" in syncer.mapping_reverse_by_url
+    assert syncer.mapping_reverse_by_url["new_repo_url"] == "new_lib"
 
 
 def test_get_pre_commit_repo_new_version_ignored() -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
-    locked_packages: dict[str, GenericLockedPackage] = {}
+    locked_packages: dict[str, GenericLockedPackage] = {"lib-name": GenericLockedPackage("lib-name", "2.0.0")}
     plugin_config = MagicMock(spec=SyncPreCommitLockConfig)
-    plugin_config.ignore = ["lib_name"]
+    plugin_config.ignore = ["lib-name"]
 
     syncer = SyncPreCommitHooksVersion(
         printer=printer,
@@ -261,12 +247,11 @@ def test_get_pre_commit_repo_new_version_ignored() -> None:
         locked_packages=locked_packages,
         plugin_config=plugin_config,
     )
+    syncer.mapping = {"lib-name": RepoInfo(repo="repo_url", rev="${rev}")}
 
     pre_commit_config_repo = PreCommitRepo("repo_url", "1.2.3")
-    mapping_db_repo_info = RepoInfo(repo="repo_url", rev="${rev}")
-    locked_package = GenericLockedPackage("lib_name", "2.0.0")
 
-    new_version = syncer.get_pre_commit_repo_new_version(pre_commit_config_repo, mapping_db_repo_info, locked_package)
+    new_version = syncer.get_pre_commit_repo_new_version(pre_commit_config_repo)
 
     assert new_version is None
 
@@ -274,7 +259,7 @@ def test_get_pre_commit_repo_new_version_ignored() -> None:
 def test_get_pre_commit_repo_new_version_version_match() -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
-    locked_packages: dict[str, GenericLockedPackage] = {}
+    locked_packages: dict[str, GenericLockedPackage] = {"lib-name": GenericLockedPackage("lib-name", "1.2.3")}
     plugin_config = MagicMock(spec=SyncPreCommitLockConfig)
     plugin_config.ignore = []
 
@@ -286,14 +271,41 @@ def test_get_pre_commit_repo_new_version_version_match() -> None:
     )
 
     pre_commit_config_repo = PreCommitRepo("repo_url", "1.2.3")
-    mapping_db_repo_info = RepoInfo(repo="repo_url", rev="${rev}")
-    locked_package = GenericLockedPackage("lib_name", "1.2.3")
+    syncer.mapping = {"lib-name": RepoInfo(repo="repo_url", rev="${rev}")}
 
-    new_version = syncer.get_pre_commit_repo_new_version(pre_commit_config_repo, mapping_db_repo_info, locked_package)
+    new_version = syncer.get_pre_commit_repo_new_version(pre_commit_config_repo)
 
     assert new_version is None
 
 
+@pytest.mark.parametrize(
+    "dependency, expected",
+    [
+        pytest.param("dep==1.2.3", "dep==1.2.3", id="same"),
+        pytest.param("dep", "dep==1.2.3", id="locked"),
+        pytest.param("other", "other", id="not-in-lock"),
+        pytest.param("dep<>unparsable", "dep<>unparsable", id="unparsable"),
+        pytest.param("dep==1.0.0+dev", "dep==1.0.0+dev", id="local"),
+        pytest.param("Dep", "Dep==1.2.3", id="casing"),
+    ],
+)
+def test_get_pre_commit_repo_hook_new_dependency(dependency: str, expected: str) -> None:
+    printer = MagicMock(spec=Printer)
+    pre_commit_config_file_path = MagicMock(spec=Path)
+    locked_packages: dict[str, GenericLockedPackage] = {"dep": GenericLockedPackage("dep", "1.2.3")}
+    plugin_config = MagicMock(spec=SyncPreCommitLockConfig)
+    plugin_config.ignore = []
+
+    syncer = SyncPreCommitHooksVersion(
+        printer=printer,
+        pre_commit_config_file_path=pre_commit_config_file_path,
+        locked_packages=locked_packages,
+        plugin_config=plugin_config,
+    )
+
+    assert syncer.get_pre_commit_repo_hook_new_dependency(dependency) == expected
+
+
 def test_analyze_repos_repo_not_in_mapping() -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
@@ -308,10 +320,9 @@ def test_analyze_repos_repo_not_in_mapping() -> None:
     )
 
     pre_commit_repos = {PreCommitRepo("repo_url", "1.2.3")}
-    mapping: PackageRepoMapping = {}
-    mapping_reverse_by_url: dict[str, str] = {}
+    syncer.mapping = {}
 
-    result, _ = syncer.analyze_repos(pre_commit_repos, mapping, mapping_reverse_by_url)
+    result, _ = syncer.analyze_repos(pre_commit_repos)
 
     assert result == {}
 
@@ -330,10 +341,9 @@ def test_analyze_repos_dependency_not_locked() -> None:
     )
 
     pre_commit_repos = {PreCommitRepo("repo_url", "1.2.3")}
-    mapping: PackageRepoMapping = {"lib_name": {"repo": "repo_url", "rev": "${rev}"}}
-    mapping_reverse_by_url = {"repo_url": "lib_name"}
+    syncer.mapping = {"lib-name": {"repo": "repo_url", "rev": "${rev}"}}
 
-    result, _ = syncer.analyze_repos(pre_commit_repos, mapping, mapping_reverse_by_url)
+    result, _ = syncer.analyze_repos(pre_commit_repos)
 
     assert result == {}
 
@@ -341,7 +351,7 @@ def test_analyze_repos_dependency_not_locked() -> None:
 def test_analyze_repos_no_new_version() -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
-    locked_packages: dict[str, GenericLockedPackage] = {"lib_name": MagicMock(version="1.2.3")}
+    locked_packages: dict[str, GenericLockedPackage] = {"lib-name": MagicMock(version="1.2.3")}
     plugin_config = MagicMock(spec=SyncPreCommitLockConfig)
     plugin_config.ignore = []
 
@@ -353,10 +363,9 @@ def test_analyze_repos_no_new_version() -> None:
     )
 
     pre_commit_repos = {PreCommitRepo("repo_url", "1.2.3")}
-    mapping = {"lib_name": RepoInfo(repo="repo_url", rev="${rev}")}
-    mapping_reverse_by_url = {"repo_url": "lib_name"}
+    syncer.mapping = {"lib-name": RepoInfo(repo="repo_url", rev="${rev}")}
 
-    result, _ = syncer.analyze_repos(pre_commit_repos, mapping, mapping_reverse_by_url)
+    result, _ = syncer.analyze_repos(pre_commit_repos)
 
     assert result == {}
 
@@ -364,7 +373,7 @@ def test_analyze_repos_no_new_version() -> None:
 def test_analyze_repos_local() -> None:
     printer = MagicMock(spec=Printer)
     pre_commit_config_file_path = MagicMock(spec=Path)
-    locked_packages: dict[str, GenericLockedPackage] = {"lib_name": MagicMock(version="0.1.1+dev")}
+    locked_packages: dict[str, GenericLockedPackage] = {"lib-name": MagicMock(version="0.1.1+dev")}
     plugin_config = MagicMock(spec=SyncPreCommitLockConfig)
     plugin_config.ignore = []
 
@@ -376,9 +385,128 @@ def test_analyze_repos_local() -> None:
     )
 
     pre_commit_repos = {PreCommitRepo("repo_url", "1.2.3")}
-    mapping = {"lib_name": RepoInfo(repo="repo_url", rev="${rev}")}
-    mapping_reverse_by_url = {"repo_url": "lib_name"}
+    syncer.mapping = {"lib-name": RepoInfo(repo="repo_url", rev="${rev}")}
 
-    result, _ = syncer.analyze_repos(pre_commit_repos, mapping, mapping_reverse_by_url)
+    result, _ = syncer.analyze_repos(pre_commit_repos)
 
     assert result == {}
+
+
+def test_analyze_repos_additional_dependencies() -> None:
+    printer = MagicMock(spec=Printer)
+    pre_commit_config_file_path = MagicMock(spec=Path)
+    locked_packages: dict[str, GenericLockedPackage] = {"lib-name": GenericLockedPackage("lib-name", "2.0.0")}
+    plugin_config = SyncPreCommitLockConfig()
+
+    syncer = SyncPreCommitHooksVersion(
+        printer=printer,
+        pre_commit_config_file_path=pre_commit_config_file_path,
+        locked_packages=locked_packages,
+        plugin_config=plugin_config,
+    )
+    pre_commit_repo = PreCommitRepo("https://repo_url", "1.2.3", [PreCommitHook("hook", ["lib-name==1.2.2"])])
+    pre_commit_repos = {pre_commit_repo}
+    syncer.mapping = {"lib-name": {"repo": "https://repo_url", "rev": "${rev}"}}
+
+    to_fix, _ = syncer.analyze_repos(pre_commit_repos)
+
+    assert to_fix == {
+        pre_commit_repo: PreCommitRepo("https://repo_url", "2.0.0", [PreCommitHook("hook", ["lib-name==2.0.0"])])
+    }
+
+
+def test_analyze_repos_not_in_lock_but_additional_dependencies() -> None:
+    printer = MagicMock(spec=Printer)
+    pre_commit_config_file_path = MagicMock(spec=Path)
+    locked_packages: dict[str, GenericLockedPackage] = {"lib-name": GenericLockedPackage("lib-name", "2.0.0")}
+    plugin_config = SyncPreCommitLockConfig()
+
+    syncer = SyncPreCommitHooksVersion(
+        printer=printer,
+        pre_commit_config_file_path=pre_commit_config_file_path,
+        locked_packages=locked_packages,
+        plugin_config=plugin_config,
+    )
+    pre_commit_repo = PreCommitRepo("https://repo_url", "1.2.3", [PreCommitHook("hook", ["lib-name==1.2.2"])])
+    pre_commit_repos = {pre_commit_repo}
+    syncer.mapping = {"not_lib": {"repo": "https://repo_url", "rev": "${rev}"}}
+
+    to_fix, _ = syncer.analyze_repos(pre_commit_repos)
+
+    assert to_fix == {
+        pre_commit_repo: PreCommitRepo("https://repo_url", "1.2.3", [PreCommitHook("hook", ["lib-name==2.0.0"])])
+    }
+
+
+def test_analyze_repos_local_but_additional_dependencies() -> None:
+    printer = MagicMock(spec=Printer)
+    pre_commit_config_file_path = MagicMock(spec=Path)
+    locked_packages: dict[str, GenericLockedPackage] = {
+        "lib-name": GenericLockedPackage("lib-name", "2.0.0"),
+        "local_lib": GenericLockedPackage("local_lib", "1.0.0+dev"),
+    }
+    plugin_config = SyncPreCommitLockConfig()
+
+    syncer = SyncPreCommitHooksVersion(
+        printer=printer,
+        pre_commit_config_file_path=pre_commit_config_file_path,
+        locked_packages=locked_packages,
+        plugin_config=plugin_config,
+    )
+    pre_commit_repo = PreCommitRepo("https://repo_url", "1.2.3", [PreCommitHook("hook", ["lib-name==1.2.2"])])
+    pre_commit_repos = {pre_commit_repo}
+    syncer.mapping = {"local_lib": {"repo": "https://repo_url", "rev": "${rev}"}}
+
+    to_fix, _ = syncer.analyze_repos(pre_commit_repos)
+
+    assert to_fix == {
+        pre_commit_repo: PreCommitRepo("https://repo_url", "1.2.3", [PreCommitHook("hook", ["lib-name==2.0.0"])])
+    }
+
+
+MOCK_DEP_MAPPING = {"dep": {"repo": "https://some.place", "rev": "${dev}"}}
+MOCK_REPO_ALIASES = {"https://some.place": ("https://some.old.place",)}
+
+
+@patch("sync_pre_commit_lock.actions.sync_hooks.DEPENDENCY_MAPPING", MOCK_DEP_MAPPING)
+@patch("sync_pre_commit_lock.actions.sync_hooks.REPOSITORY_ALIASES", MOCK_REPO_ALIASES)
+def test_analyze_repos_renamed() -> None:
+    printer = MagicMock(spec=Printer)
+    pre_commit_config_file_path = MagicMock(spec=Path)
+    locked_packages: dict[str, GenericLockedPackage] = {"ruff": GenericLockedPackage("dep", "1.2.3")}
+    plugin_config = SyncPreCommitLockConfig()
+
+    syncer = SyncPreCommitHooksVersion(
+        printer=printer,
+        pre_commit_config_file_path=pre_commit_config_file_path,
+        locked_packages=locked_packages,
+        plugin_config=plugin_config,
+    )
+    pre_commit_repo = PreCommitRepo("https://some.old.place", "1.2.3")
+    pre_commit_repos = {pre_commit_repo}
+
+    to_fix, _ = syncer.analyze_repos(pre_commit_repos)
+
+    assert to_fix == {pre_commit_repo: PreCommitRepo("https://some.place", "1.2.3")}
+
+
+@patch("sync_pre_commit_lock.actions.sync_hooks.DEPENDENCY_MAPPING", MOCK_DEP_MAPPING)
+@patch("sync_pre_commit_lock.actions.sync_hooks.REPOSITORY_ALIASES", MOCK_REPO_ALIASES)
+def test_analyze_repos_already_last_url() -> None:
+    printer = MagicMock(spec=Printer)
+    pre_commit_config_file_path = MagicMock(spec=Path)
+    locked_packages: dict[str, GenericLockedPackage] = {"ruff": GenericLockedPackage("dep", "1.2.3")}
+    plugin_config = SyncPreCommitLockConfig()
+
+    syncer = SyncPreCommitHooksVersion(
+        printer=printer,
+        pre_commit_config_file_path=pre_commit_config_file_path,
+        locked_packages=locked_packages,
+        plugin_config=plugin_config,
+    )
+    pre_commit_repo = PreCommitRepo("https://some.place", "1.2.3")
+    pre_commit_repos = {pre_commit_repo}
+
+    to_fix, _ = syncer.analyze_repos(pre_commit_repos)
+
+    assert to_fix == {}
